/*
 * Copyright 2019 Collabora, Ltd.
 *
 * SPDX-License-Identifier: MIT
 */
/*
 * This program should be passed a usb device node matching an Atari controller
 * in xbox 360 compatibility mode. THe controller will then be switched to
 * atari mode.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include <libusb.h>

#define INTERFACE 0
#define ENDPOINT 0x2
#define PACKET_LEN 3

#define VID 0x45e
#define PID 0x28e
#define MIN_VERSION 0x135

#define FAIL_IF(expr, ...)             \
  do {                                 \
    if (expr) {                        \
        fprintf (stderr, __VA_ARGS__); \
        exit(1);                       \
    }                                  \
  } while (0)

int
main (int argc, char *argv[])
{
  int ret;
  libusb_device_handle *h = NULL;
  libusb_device *d = NULL;
  struct libusb_device_descriptor desc;
  /* LED packet, 3 bytes length */
  uint8_t data[PACKET_LEN] = { 0x1, 0x3, } ;
  char *cmds = "Atari";
  int fd;

  FAIL_IF (argc != 2, "Incorrect number of arguments\n");

  ret = libusb_init (NULL);
  FAIL_IF (ret != 0, "Couldn't initialize libusb\n");

  fd = open(argv[1], O_RDWR);
  FAIL_IF (fd < 0, "Failed to open device: %s", strerror (errno));

  ret = libusb_wrap_sys_device (NULL, fd, &h);
  FAIL_IF (ret != 0 || h == NULL, "Failed to wrap fd for libusb\n");

  d = libusb_get_device (h);
  FAIL_IF (d == NULL, "Failed to get usb device\n");

  ret = libusb_get_device_descriptor (d, &desc);
  FAIL_IF (ret != 0, "Failed to get device descriptor\n");

  FAIL_IF (desc.idVendor != VID,
    "Unsupported vendor: 0x%04x\n", desc.idVendor);
  FAIL_IF (desc.idProduct != PID,
    "Unsupported product: 0x%04x\n", desc.idProduct);
  FAIL_IF (desc.bcdDevice <  MIN_VERSION,
    "Version too old: %x\n", desc.bcdDevice);

  libusb_detach_kernel_driver (h, INTERFACE);
  ret = libusb_claim_interface(h, INTERFACE);

  for (int i = 0; cmds[i] != '\0' ; i++) {
    data[2] = cmds[i];

    ret = libusb_interrupt_transfer (h,
      ENDPOINT,
      data,
      PACKET_LEN,
      NULL,
      2000);

    FAIL_IF (ret != 0, "Failed to send command to device: %d\n", ret);
  }

  return 0;
}
